A build of the package {{ details.name }} has failed against the current master[1] of
the upstream project, please see log[2] and update the packaging[3].

You are receiving this email because you are listed as one of the
maintainers for the {{ details.name }}.

[1] - {{ details.upstream }}
[2] - {{ details.logurl }}
[3] - {{ details['master-distgit'] }}
