@import url(https://unpkg.com/@patternfly/patternfly/patternfly.css);
@import url(https://cdn.datatables.net/1.10.22/css/jquery.dataTables.min.css);

#dlrn tr.failure td {
    color: #000000;
    background-color: #f5c6cb;
}

#dlrn tr.failure:hover td {
    background-color: #f1b0b7;
}

#dlrn th {
    color: #ffffff;
    background-color: #550000;
}

h1 {
    color: #7F0A0C;
    margin-top: 2.25rem;
    margin-bottom: 1em;
    font-size: 2em;
}

