# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


class BuildRPMDriver(object):
    """Abstract base class for Build RPM drivers

    """
    def __init__(self, *args, **kwargs):
        self.config_options = kwargs.get('cfg_options')

    def build_package(self):
        """Perform package build

        """
        return False
